//
//  SysInfo.m - info on the system memory usage at one point in time
//  VMTester
//
//  Created by Cameron Hayne on 30/11/04.
//  Copyright 2006 Hayne of Tintagel.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#import "SysInfo.h"

// utility functions
static bool getVmStat(vm_statistics_data_t *ptrVmStat)
{
    unsigned count = HOST_VM_INFO_COUNT;
    kern_return_t result = host_statistics(mach_host_self(),
                                           HOST_VM_INFO,
                                           (host_info_t) ptrVmStat,
                                           &count);
    if (result == KERN_SUCCESS)
    {
        return true;
    }
    else
    {
        NSLog(@"VMTester:getVMStat: host_statistics failed (returned %d)", result);
        return false;
    }
}

static bool getSwapUsage(struct xsw_usage *ptrSwapUsage)
{
    // based on the code in Darwin's sysctl.c
    int mib[CTL_MAXNAME];
    mib[0] = CTL_VM;
    mib[1] = VM_SWAPUSAGE;
    size_t size = sizeof(struct xsw_usage);
    if (sysctl(mib, 2, ptrSwapUsage, &size, NULL, 0) == 0)
    {
        return true;
    }
    else
    {
        NSLog(@"VMTester:getSwapUsage: sysctl failed: %s", strerror(errno));
        return false;
    }
}

@implementation SysInfo

static unsigned long thePageInsBaseline = 0;
static unsigned long thePageOutsBaseline = 0;

// class method to reset the above baselines to the current values
+ (void)resetPageInsOutsBaselines
{
    vm_statistics_data_t vmStat;
    if (getVmStat(&vmStat))
    {
        thePageInsBaseline = vmStat.pageins;
        thePageOutsBaseline = vmStat.pageouts;
    }
}

- (id)init
{
    bool status;
    myPageInsBaseline = thePageInsBaseline;
    myPageOutsBaseline = thePageOutsBaseline;

    status = getVmStat(&myVmStat);
    
    if (status)
    {
        status = getSwapUsage(&mySwapUsage);
    }
    
    return status ? self : nil;
}

- (void)dealloc
{
    [super dealloc];
}

@end
