//
//  SysInfoCtr.m  - handles the display of the "System Information" table
//  VMTester
//
//  Created by Cameron Hayne on 19/02/05.
//  Copyright 2006 Hayne of Tintagel.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#import "SysInfoCtr.h"

#define BYTES_PER_MB (1024.0 * 1024.0)

// private SysInfoCtr methods
@interface SysInfoCtr (Private)
- (NSString *)strMB:(long)numPages;
- (NSString *)strNum:(long)numPages;
- (int)numPageIns:(int)pageins;
- (int)numPageOuts:(int)pageouts;
@end

@implementation SysInfoCtr

- (id)init
{
    vm_size_t pageSize;
    host_page_size(mach_host_self(), &pageSize);
    myPageSizeMB = pageSize / BYTES_PER_MB;
    
    return self;
}

- (void)dealloc
{
    [super dealloc];
}

- (void)setCurrentInfo:(SysInfo *)curr PrevInfo:(SysInfo *)prev
{
    myCurrSysInfo = curr;
    myPrevSysInfo = prev;
}

// utility method
- (NSString *)bytesStrMB:(long long)numBytes
{
    return [NSString stringWithFormat: @"%.1f", numBytes / BYTES_PER_MB];
}

// utility method
- (NSString *)pagesStrMB:(long)numPages
{
    return [NSString stringWithFormat: @"%.1f", numPages * myPageSizeMB];
}

// utility method
- (NSString *)pagesStrNum:(long)numPages
{
    return [NSString stringWithFormat: @"%ld", numPages];
}

// utility method
- (int)numPageIns:(int)pageins
{
    return (pageins - myCurrSysInfo->myPageInsBaseline);
}

// utility method
- (int)numPageOuts:(int)pageouts
{
    return (pageouts - myCurrSysInfo->myPageOutsBaseline);
}

// used for exporting the data
- (NSString *)commaSeparatedLegend
{
    NSString *str = [NSString stringWithFormat: @"%@,%@,%@,%@,%@,%@,%@,%@,%@,%@",
                    @"Wired (MB)",
                    @"Active (MB)",
                    @"Inactive (MB)",
                    @"Free (MB)",
                    @"Pageins",
                    @"Pageins (MB)",
                    @"Pageouts",
                    @"Pageouts (MB)",
                    @"Swap Total (MB)",
                    @"Swap Used (MB)"
                    ];
    return str;
}

// used for exporting the data
- (NSString *)sysInfoCommaSeparated:(SysInfo *)sysInfo
{
    vm_statistics_data_t *currVm = &sysInfo->myVmStat;
    struct xsw_usage     *currSw = &sysInfo->mySwapUsage;
    NSString *str = [NSString stringWithFormat: @"%@,%@,%@,%@,%@,%@,%@,%@,%@,%@",
                    [self pagesStrMB:  currVm->wire_count],
                    [self pagesStrMB:  currVm->active_count],
                    [self pagesStrMB:  currVm->inactive_count],
                    [self pagesStrMB:  currVm->free_count],
                    [self pagesStrNum: [self numPageIns:currVm->pageins]],
                    [self pagesStrMB:  [self numPageIns:currVm->pageins]],
                    [self pagesStrNum: [self numPageOuts:currVm->pageouts]],
                    [self pagesStrMB:  [self numPageOuts:currVm->pageouts]],
                    [self bytesStrMB:  currSw->xsu_total],
                    [self bytesStrMB:  currSw->xsu_used]
                    ];
    return str;
}

// number of rows in the "System Information" table
- (int)numberOfRowsInTableView:(NSTableView *)tableView
{
    return 10;
}

// values in the "System Information" table
- (id)tableView:(NSTableView *)tableView
      objectValueForTableColumn:(NSTableColumn *)tableColumn
            row:(int)row
{
    NSString * colId = [tableColumn identifier];
    if ([colId compare: @"stat"] == NSOrderedSame)
    {
        switch (row)
        {
            case  0: return @"Wired (MB)";
            case  1: return @"Active (MB)";
            case  2: return @"Inactive (MB)";
            case  3: return @"Free (MB)";
            case  4: return @"Pageins";
            case  5: return @"Pageins (MB)";
            case  6: return @"Pageouts";
            case  7: return @"Pageouts (MB)";
            case  8: return @"Swap Total (MB)";
            case  9: return @"Swap Used (MB)";
        }
    }
    else if ([colId compare: @"value"] == NSOrderedSame)
    {
        if (myCurrSysInfo == nil)
        {
            return @"0";
        }
        
        vm_statistics_data_t *currVm = &myCurrSysInfo->myVmStat;
        struct xsw_usage     *currSw = &myCurrSysInfo->mySwapUsage;
        switch (row)
        {
            case  0: return [self pagesStrMB:  currVm->wire_count];
            case  1: return [self pagesStrMB:  currVm->active_count];
            case  2: return [self pagesStrMB:  currVm->inactive_count];
            case  3: return [self pagesStrMB:  currVm->free_count];
            case  4: return [self pagesStrNum: [self numPageIns:currVm->pageins]];
            case  5: return [self pagesStrMB:  [self numPageIns:currVm->pageins]];
            case  6: return [self pagesStrNum: [self numPageOuts:currVm->pageouts]];
            case  7: return [self pagesStrMB:  [self numPageOuts:currVm->pageouts]];
            case  8: return [self bytesStrMB:  currSw->xsu_total];
            case  9: return [self bytesStrMB:  currSw->xsu_used];
        }
    }
    else // delta
    {
        if (myCurrSysInfo == nil || myPrevSysInfo == nil)
        {
            return @"0";
        }
        
        vm_statistics_data_t *currVm = &myCurrSysInfo->myVmStat;
        vm_statistics_data_t *prevVm = &myPrevSysInfo->myVmStat;
        struct xsw_usage     *currSw = &myCurrSysInfo->mySwapUsage;
        struct xsw_usage     *prevSw = &myPrevSysInfo->mySwapUsage;
        switch (row)
        {
            case  0: return [self pagesStrMB:  currVm->wire_count - prevVm->wire_count];
            case  1: return [self pagesStrMB:  currVm->active_count - prevVm->active_count];
            case  2: return [self pagesStrMB:  currVm->inactive_count - prevVm->inactive_count];
            case  3: return [self pagesStrMB:  currVm->free_count - prevVm->free_count];
            case  4: return [self pagesStrNum: currVm->pageins - prevVm->pageins];
            case  5: return [self pagesStrMB:  currVm->pageins - prevVm->pageins];
            case  6: return [self pagesStrNum: currVm->pageouts - prevVm->pageouts];
            case  7: return [self pagesStrMB:  currVm->pageouts - prevVm->pageouts];
            case  8: return [self bytesStrMB:  currSw->xsu_total - prevSw->xsu_total];
            case  9: return [self bytesStrMB:  currSw->xsu_used - prevSw->xsu_used];
        }
    }
}

- (float)pageSizeMB
{
    return myPageSizeMB;
}

@end
